
Imports System
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Collections
Imports System.Collections.Specialized
Imports System.Data
Imports System.Data.SqlClient
Imports System.Drawing


Namespace Superexpert


'*********************************************************************
'
' SuperDataGrid Class
'
' The SuperDataGrid control supports sorting, paging, editing, 
' and deleting records through its
' EnableSorting, EnablePaging, EnableEditing, and EnableDeleting 
' properties.
'
' See the SuperDataGridCS.aspx or SuperDataGridVB.aspx files for
' samples of using this control.
'
' This control is from the book ASP.NET Unleashed written by 
' Stephhen Walther and published by SAMS publishing.
'
'*********************************************************************

Public Class SuperDataGrid
    Inherits WebControl
    Implements IPostBackDataHandler, IPostBackEventHandler 
    
    Private _enableSorting As Boolean = True
    Private _enableEditing As Boolean = True
    Private _enableDeleting As Boolean = True
    Private _enablePaging As Boolean = True
    
    Private _dataGridData As New DataTable()
    Private _dataGridView As New DataView()
    Private _dataGridSchema As New DataTable()
    Private colEditValues As New Hashtable()
    
    Private _editClicked As Boolean = False
    Private _updateClicked As Boolean = False
    Private _deleteClicked As Boolean = False
    Private _deleteItemIndex As Integer = - 1
    
    Private _tableStyle As New TableStyle()
    Private _headerStyle As New TableItemStyle()
    Private _itemStyle As New TableItemStyle()
    Private _alternatingItemStyle As New TableItemStyle()
    Private _pagerStyle As New TableItemStyle()
    
    Private _commandText As String = String.Empty
    Private _pageSize As Integer = 10
    Private _prevText As String = "&lt;"
    Private _nextText As String = "&gt;"
    Private _pagerText As String = "Page {0} of {1}"
    
    Private _errorMessage As String = String.Empty
    
    
    '*********************************************************************
    '
    ' ConnectionString Property
    '
    ' The Connection String for the database. Notice that this must
    ' be a valid SQL connection string since the SuperDataGrid uses
    ' the SqlClient classes.
    '
    '*********************************************************************
    
    Public Property ConnectionString() As String
        Get
            If ViewState("ConnectionString") Is Nothing Then
                Return String.Empty
            Else
                Return CStr(ViewState("ConnectionString"))
            End If
        End Get
        Set
            ViewState("ConnectionString") = value
        End Set
    End Property 
    
    
    '*********************************************************************
    '
    ' TableName Property
    '
    ' The name of the database table to retrieve the data from. Instead
    ' of using this property, you can use the CommandText property and
    ' specify your own SELECT statement (with parameters).
    '
    '*********************************************************************
    
    Public Property TableName() As String
        Get
            If ViewState("TableName") Is Nothing Then
                Return String.Empty
            Else
                Return CStr(ViewState("TableName"))
            End If
        End Get
        Set
            ViewState("TableName") = value
        End Set
    End Property 
    
    
    '*********************************************************************
    '
    ' CommandText Property
    '
    ' The CommandText property enables you to specify a SELECT statement
    ' to use when retrieving database data.
    '
    '*********************************************************************
    
    Public Property CommandText() As String
        Get
            If ViewState("CommandText") Is Nothing Then
                Return String.Empty
            Else
                Return CStr(ViewState("CommandText"))
            End If
        End Get
        Set
            ViewState("CommandText") = value
        End Set
    End Property 
    
    '*********************************************************************
    '
    ' TableStyle Property
    '
    ' The TableStyle object applied to the SuperDataGrid table. Use
    ' this property to control the formatting of the main table.
    '
    '*********************************************************************
    
    Public ReadOnly Property TableStyle() As TableStyle
        Get
            Return _tableStyle
        End Get
    End Property 
    
    
    '*********************************************************************
    '
    ' HeaderStyle Property
    '
    ' Use this property to control the formatting of the Header row.
    '
    '*********************************************************************
    
    Public ReadOnly Property HeaderStyle() As TableItemStyle
        Get
            Return _headerStyle
        End Get
    End Property 
    
    
    '*********************************************************************
    '
    ' ItemStyle Property
    '
    ' Use this property to control the formatting of each row.
    '
    '*********************************************************************
    
    Public ReadOnly Property ItemStyle() As TableItemStyle
        Get
            Return _itemStyle
        End Get
    End Property 
    
    
    '*********************************************************************
    '
    ' AlternatingItemStyle Property
    '
    ' Use this property to control the formatting of every other row.
    '
    '*********************************************************************
    
    Public ReadOnly Property AlternatingItemStyle() As TableItemStyle
        Get
            Return _alternatingItemStyle
        End Get
    End Property 
    
    
    '*********************************************************************
    '
    ' PagerStyle Property
    '
    ' Use this property to control the formatting of the pager row.
    '
    '*********************************************************************
    
    Public ReadOnly Property PagerStyle() As TableItemStyle
        Get
            Return _pagerStyle
        End Get
    End Property 
    
    '*********************************************************************
    '
    ' SortColumn Property
    '
    ' The column to sort on.
    '
    '*********************************************************************
    
    Public Property SortColumn() As String
        Get
            If ViewState("SortColumn") Is Nothing Then
                Return String.Empty
            Else
                Return CStr(ViewState("SortColumn"))
            End If
        End Get
        Set
            ViewState("SortColumn") = value
        End Set
    End Property 
    
    
    '*********************************************************************
    '
    ' SortOrder Property
    '
    ' The Order to sort by (for example, ASC or DESC).
    '
    '*********************************************************************
    
    Public Property SortOrder() As String
        Get
            If ViewState("SortOrder") Is Nothing Then
                Return "ASC"
            Else
                Return CStr(ViewState("SortOrder"))
            End If
        End Get
        Set
            ViewState("SortOrder") = value
        End Set
    End Property 
    
    
    '*********************************************************************
    '
    ' CurrentPageIndex Property
    '
    ' The current page of records to display.
    '
    '*********************************************************************
    
    Public Property CurrentPageIndex() As Integer
        Get
            If ViewState("CurrentPageIndex") Is Nothing Then
                Return 1
            Else
                Return ViewState("CurrentPageIndex")
            End If
        End Get
        Set
            ViewState("CurrentPageIndex") = value
        End Set
    End Property 
    
    
    '*********************************************************************
    '
    ' EnableSorting Property
    '
    ' When true, columns can be sorted.
    '
    '*********************************************************************
    
    Public Property EnableSorting() As Boolean
        Get
            Return _enableSorting
        End Get
        Set
            _enableSorting = value
        End Set
    End Property 
    
    
    
    '*********************************************************************
    '
    ' EnableEditing Property
    '
    ' When true, rows can be edited.
    '
    '*********************************************************************
    
    Public Property EnableEditing() As Boolean
        Get
            Return _enableEditing
        End Get
        Set
            _enableEditing = value
        End Set
    End Property 
    
    '*********************************************************************
    '
    ' EnableDeleting Property
    '
    ' When true, rows can be deleted.
    '
    '*********************************************************************
    
    Public Property EnableDeleting() As Boolean
        Get
            Return _enableDeleting
        End Get
        Set
            _enableDeleting = value
        End Set
    End Property 
    
    
    
    '*********************************************************************
    '
    ' PageSize Property
    '
    ' The number of rows to display per page when paging is enabled.
    '
    '*********************************************************************
    
    Public Property PageSize() As Integer
        Get
            Return _pageSize
        End Get
        Set
            _pageSize = value
        End Set
    End Property 
    
    
    '*********************************************************************
    '
    ' EditItemIndex Property
    '
    ' The current row selected for editing.
    '
    '*********************************************************************
    
    Public Property EditItemIndex() As Integer
        Get
            If ViewState("EditItemIndex") Is Nothing Then
                Return - 1
            Else
                Return ViewState("EditItemIndex")
            End If
        End Get
        Set
            ViewState("EditItemIndex") = value
        End Set
    End Property
     
    
    
    '*********************************************************************
    '
    ' LoadPostData Method
    '
    ' When editing is enabled, and data is submitted, this method
    ' updates the data stored in the colEditValues collection.
    '
    '*********************************************************************
    Public Function LoadPostData(postDataKey As String, values As NameValueCollection) As Boolean Implements IPostBackDataHandler.LoadPostData
        Dim key As String
        For Each key In  values.Keys
            colEditValues(key) = values(key)
        Next key
        Return False
    End Function 'LoadPostData
    
    
    
    '*********************************************************************
    '
    ' RaisePostDataChangedEvent Method
    '
    ' Required by the PostBackDataHandler interface.
    '
    '*********************************************************************
    Public Sub RaisePostDataChangedEvent() Implements IPostBackDataHandler.RaisePostDataChangedEvent
    End Sub 'RaisePostDataChangedEvent
    
    
    
    
    '*********************************************************************
    '
    ' RaisePostBackEvent Method
    '
    ' When a link is clicked (for sorting, paging, editing), this
    ' method launches off the right action.
    '
    '*********************************************************************
    Public Sub RaisePostBackEvent(eventArgument As String) Implements IPostBackEventHandler.RaisePostBackEvent
        Dim parts As String() = eventArgument.Split(New [Char]() {"_"c}, 2)
        
        Select Case parts(0)
            Case "edit"
                _editClicked = True
                EditItemIndex = Int32.Parse(parts(1))
            Case "cancel"
                EditItemIndex = - 1
            Case "update"
                _updateClicked = True
            Case "sort"
                UpdateSort(parts(1))
            Case "page"
                UpdatePage(Int32.Parse(parts(1)))
            Case "delete"
                _deleteClicked = True
                _deleteItemIndex = Int32.Parse(parts(1))
        End Select
    End Sub 'RaisePostBackEvent
    
    
    
    
    '*********************************************************************
    '
    ' UpdatePage Method
    '
    ' Change the current page when paging is enabled.
    '
    '*********************************************************************
    Private Sub UpdatePage(newPageIndex As Integer)
        CurrentPageIndex = newPageIndex
    End Sub 'UpdatePage
    
    
    
    
    '*********************************************************************
    '
    ' UpdateDatabase Method
    '
    ' Perform the update to the database table.
    '
    '*********************************************************************
    Private Sub UpdateDatabase()
        ' Start by updating the data table
        Dim row As DataRow = _dataGridView(EditItemIndex).Row
        Dim col As DataColumn
        For Each col In  _dataGridData.Columns
            If colEditValues.Contains("txt" & col.ColumnName) Then
                Try
                    row(col.ColumnName) = colEditValues(("txt" & col.ColumnName))
                Catch 
                    _errorMessage = "Could not update column " & col.ColumnName
                    Return
                End Try
            End If
        Next col 
        ' Create a command builder
        Dim dad As New SqlDataAdapter(_commandText, ConnectionString)
        Dim builder As New SqlCommandBuilder(dad)
        Try
            dad.Update(_dataGridData)
        Catch ex As Exception
            _errorMessage = ex.Message
            Return
        End Try
        
        ' Deselect any row for editing
        EditItemIndex = - 1
    End Sub 'UpdateDatabase
    
    
    
    Private Sub DeleteRow(rowIndex As Integer)
        ' Delete row from datatable
        Dim row As DataRow = _dataGridView(rowIndex).Row
        row.Delete()
        
        ' Create a command builder
        Dim dad As New SqlDataAdapter(_commandText, ConnectionString)
        Dim builder As New SqlCommandBuilder(dad)
        Try
            dad.Update(_dataGridData)
        Catch ex As Exception
            _errorMessage = ex.Message
        End Try
    End Sub 'DeleteRow
    
    
    
    '*********************************************************************
    '
    ' UpdateSort Method
    '
    ' When a new column is clicked for sorting, update the SortColumn
    ' and SortOrder.
    '
    '*********************************************************************
    Private Sub UpdateSort(sortExpression As String)
        EditItemIndex = - 1
        CurrentPageIndex = 1
        If sortExpression = SortColumn Then
            If SortOrder = "DESC" Then
                SortOrder = "ASC"
            Else
                SortOrder = "DESC"
            End If
        Else
            SortOrder = "ASC"
        End If 
        SortColumn = sortExpression
    End Sub 'UpdateSort
    
    
    
    
    '*********************************************************************
    '
    ' InitDatabaseObjects Method
    '
    ' Initialize the connection string and the command text.
    '
    '*********************************************************************
    Private Sub InitDatabaseObjects()
        ' Check connection string
        If ConnectionString = String.Empty Then
            Throw New ArgumentException("You must set the ConnectionString property!")
        End If 
        ' Check table name
        If TableName = String.Empty AndAlso CommandText = String.Empty Then
            Throw New ArgumentException("You must set the TableName or CommandText property!")
        End If 
        ' Calculate SELECT statement
        If CommandText = String.Empty Then
            _commandText = String.Format("SELECT * FROM {0}", TableName)
        Else
            _commandText = CommandText
        End If
    End Sub 'InitDatabaseObjects
     
    
    
    '*********************************************************************
    '
    ' LoadDatabaseData Method
    '
    ' Load the database data and schema.
    '
    '*********************************************************************
    Private Sub LoadDatabaseData()
        InitDatabaseObjects()
        
        ' Get datagrid table
        Dim con As New SqlConnection(ConnectionString)
        con.Open()
        Dim dad As New SqlDataAdapter(_commandText, con)
        dad.Fill(_dataGridData)
        
        
        ' Get datagrid schema
        Dim cmd As New SqlCommand(_commandText, con)
        _dataGridSchema = cmd.ExecuteReader(CommandBehavior.SchemaOnly).GetSchemaTable()
        
        con.Close()
    End Sub 'LoadDatabaseData
    
    
    
    
    
    '*********************************************************************
    '
    ' RenderTable Method
    '
    ' Display the main HTML table to display the database data.
    '
    '*********************************************************************
    Private Sub RenderTable(writer As HtmlTextWriter)
        
        ' Calculate Alternating Item Style
        _alternatingItemStyle.MergeWith(_itemStyle)
        
        
        
        ' Calculate Start Index
        Dim startIndex As Integer = 0
        If _enablePaging Then
            startIndex =(CurrentPageIndex - 1) * _pageSize
        End If 
        ' Calculate End Index
        Dim endIndex As Integer = _dataGridData.Rows.Count
        If _enablePaging Then
            endIndex = startIndex + _pageSize
            If endIndex > _dataGridData.Rows.Count Then
                endIndex = _dataGridData.Rows.Count
            End If
        End If 
        
        ' Render rows        
        Dim i As Integer
        For i = startIndex To endIndex - 1
            If i Mod 2 = 0 Then
                _itemStyle.AddAttributesToRender(writer)
            Else
                _alternatingItemStyle.AddAttributesToRender(writer)
            End If 
            
            If _enableEditing AndAlso EditItemIndex = i Then
                RenderEditRow(writer, i)
            Else
                RenderRow(writer, i)
            End If
        Next i
    End Sub 'RenderTable
     
    
    
    '*********************************************************************
    '
    ' RenderRow Method
    '
    ' Display a particular row in the HTML table.
    '
    '*********************************************************************
    Private Sub RenderRow(writer As HtmlTextWriter, itemIndex As Integer)
        writer.RenderBeginTag(HtmlTextWriterTag.Tr)
        If _enableEditing Then
            writer.RenderBeginTag(HtmlTextWriterTag.Td)
            RenderLink(writer, "Edit", "edit_" & itemIndex.ToString())
            
            If _enableDeleting Then
                writer.Write("&nbsp;|&nbsp;")
                writer.AddAttribute(HtmlTextWriterAttribute.Onclick, "return confirm('Are you sure you want to delete this row?')")
                
                RenderLink(writer, "Delete", "delete_" & itemIndex.ToString())
            End If
            
            writer.RenderEndTag()
        End If
        
        Dim i As Integer
        For i = 0 To _dataGridData.Columns.Count - 1
            writer.RenderBeginTag(HtmlTextWriterTag.Td)
            RenderColumn(writer, itemIndex, i)
            writer.RenderEndTag()
        Next i
        writer.RenderEndTag()
    End Sub 'RenderRow
    
    
    
    
    '*********************************************************************
    '
    ' RenderEditRow Method
    '
    ' Render a row for editing.
    '
    '*********************************************************************
    Private Sub RenderEditRow(writer As HtmlTextWriter, itemIndex As Integer)
        writer.RenderBeginTag(HtmlTextWriterTag.Tr)
        writer.RenderBeginTag(HtmlTextWriterTag.Td)
        
        ' Render update link
        RenderLink(writer, "Update", "update_" & itemIndex.ToString())
        
        writer.Write("&nbsp;|&nbsp;")
        
        ' Render Cancel link
        RenderLink(writer, "Cancel", "cancel_" & itemIndex.ToString())
        
        writer.RenderEndTag()
        
        Dim i As Integer
        For i = 0 To _dataGridData.Columns.Count - 1
            writer.AddAttribute(HtmlTextWriterAttribute.Valign, "top")
            writer.RenderBeginTag(HtmlTextWriterTag.Td)
            RenderEditColumn(writer, itemIndex, i)
            writer.RenderEndTag()
        Next i
        writer.RenderEndTag()
    End Sub 'RenderEditRow
    
    
    
    
    '*********************************************************************
    '
    ' RenderColumn Method
    '
    ' Render the value of a column. For money columns, we'll apply 
    ' some formatting.
    '
    '*********************************************************************
    Private Sub RenderColumn(writer As HtmlTextWriter, rowIndex As Integer, colIndex As Integer)
        Dim colValue As Object = _dataGridView(rowIndex)(colIndex)
        Dim providerType As SqlDbType = CType(_dataGridSchema.Rows(colIndex)("ProviderType"), SqlDbType)
        
        Select Case providerType
            Case SqlDbType.Money
                writer.Write(System.Convert.ToDecimal(colValue).ToString("c"))
            Case SqlDbType.Image
                writer.Write("[Image]")
            Case SqlDbType.DateTime
                Dim dtm As DateTime = CType(colValue, DateTime)
                If dtm.Date = dtm Then
                    writer.Write(dtm.ToString("d"))
                Else
                    writer.Write(dtm.ToString("g"))
                End If
            Case Else
                writer.Write(colValue.ToString())
        End Select
    End Sub 'RenderColumn
    
    
    
    
    '*********************************************************************
    '
    ' RenderEditColumn Method
    '
    ' Render a column with the right UI for editing its contents.
    '
    '*********************************************************************
    Private Sub RenderEditColumn(writer As HtmlTextWriter, rowIndex As Integer, colIndex As Integer)
        Dim colName As String = _dataGridData.Columns(colIndex).ColumnName
        Dim colValue As Object = _dataGridView(rowIndex)(colIndex)
        If colEditValues.Contains(colName) Then
            colValue = colEditValues(colName)
        End If
        Dim providerType As SqlDbType = CType(_dataGridSchema.Rows(colIndex)("ProviderType"), SqlDbType)
        Dim columnSize As Integer = _dataGridSchema.Rows(colIndex)("ColumnSize")
        Dim colIsIdentity As Boolean = CBool(_dataGridSchema.Rows(colIndex)("IsIdentity"))
        
        If colIsIdentity Then
            writer.Write(colValue.ToString())
        Else
            Select Case providerType
                Case SqlDbType.VarChar, SqlDbType.NVarChar, SqlDbType.NText, SqlDbType.Text
                    If columnSize > 100 Then
                        RenderTextBox(writer, colName, colValue, 30, 3, columnSize)
                    ElseIf columnSize > 30 Then
                        RenderTextBox(writer, colName, colValue, 30, 1, columnSize)
                    Else
                        RenderTextBox(writer, colName, colValue, columnSize, 1, columnSize)
                    End If
                Case SqlDbType.Bit
                    RenderSelect(writer, colName)
                Case SqlDbType.Image
                    writer.Write("[Image]")
                Case SqlDbType.DateTime
                    Dim dtm As DateTime = CType(colValue, DateTime)
                    If dtm.Date = dtm Then
                        RenderTextBox(writer, colName, dtm.ToString("d"), 10, 1, 20)
                    Else
                        RenderTextBox(writer, colName, dtm.ToString("g"), 20, 1, 30)
                    End If
                Case Else
                    RenderTextBox(writer, colName, colValue, 5, 1, columnSize)
            End Select
        End If
    End Sub 'RenderEditColumn
    
    
    
    
    '*********************************************************************
    '
    ' RenderTextBox Method
    '
    ' Display a textbox.
    '
    '*********************************************************************
    Private Sub RenderTextBox(writer As HtmlTextWriter, colName As String, colValue As Object, columns As Integer, rows As Integer, maxLength As Integer)
        Dim name As String = "txt" & colName
        If rows > 1 Then
            writer.AddAttribute(HtmlTextWriterAttribute.Id, name)
            writer.AddAttribute(HtmlTextWriterAttribute.Name, name)
            writer.AddAttribute(HtmlTextWriterAttribute.Cols, columns.ToString())
            writer.AddAttribute(HtmlTextWriterAttribute.Rows, rows.ToString())
            writer.RenderBeginTag(HtmlTextWriterTag.Textarea)
            writer.Write(colValue)
            writer.RenderEndTag()
        Else
            writer.Write(String.Format("<input type=""text"" name=""{0}"" id=""{0}"" value=""{1}"" size=""{2}"" maxlength=""{3}"">", name, colValue, columns, maxLength))
        End If
    End Sub 'RenderTextBox
     
    
    '*********************************************************************
    '
    ' RenderSelect Method
    '
    ' Display an HTML select list.
    '
    '*********************************************************************
    Private Sub RenderSelect(writer As HtmlTextWriter, colName As String)
        Dim name As String = "txt" & colName
        Dim selected As Boolean = False
        If colEditValues(name).ToString() = "True" Then
            selected = True
        End If 
        writer.AddAttribute(HtmlTextWriterAttribute.Id, name)
        writer.AddAttribute(HtmlTextWriterAttribute.Name, name)
        writer.RenderBeginTag(HtmlTextWriterTag.Select)
        writer.AddAttribute(HtmlTextWriterAttribute.Value, "True")
        If selected Then
            writer.AddAttribute(HtmlTextWriterAttribute.Selected, "SELECTED")
        End If
        writer.RenderBeginTag(HtmlTextWriterTag.Option)
        writer.Write("True")
        writer.RenderEndTag()
        writer.AddAttribute(HtmlTextWriterAttribute.Value, "False")
        If Not selected Then
            writer.AddAttribute(HtmlTextWriterAttribute.Selected, "SELECTED")
        End If
        writer.RenderBeginTag(HtmlTextWriterTag.Option)
        writer.Write("False")
        writer.RenderEndTag()
        writer.RenderEndTag()
    End Sub 'RenderSelect
    
    
    
    '*********************************************************************
    '
    ' RenderLink Method
    '
    ' Display a link that causes a postback.
    '
    '*********************************************************************
    Private Sub RenderLink(writer As HtmlTextWriter, label As String, argument As String)
        writer.AddAttribute(HtmlTextWriterAttribute.Href, "javascript:" & Page.GetPostBackEventReference(Me, argument))
        writer.RenderBeginTag(HtmlTextWriterTag.A)
        writer.Write(label)
        writer.RenderEndTag()
    End Sub 'RenderLink
    
    
    
    '*********************************************************************
    '
    ' RenderHeaderRow Method
    '
    ' Display the first row in the HTML table. This row either displays
    ' just the column names or column names that can be clicked and sorted.
    '
    '*********************************************************************
    Private Sub RenderHeaderRow(writer As HtmlTextWriter)
        HeaderStyle.AddAttributesToRender(writer)
        writer.RenderBeginTag(HtmlTextWriterTag.Tr)
        If _enableEditing Then
            writer.RenderBeginTag(HtmlTextWriterTag.Td)
            writer.Write("&nbsp;")
            writer.RenderEndTag()
        End If
        
        If _enableSorting Then
            RenderSortHeaderRow(writer)
        Else
            RenderNormalHeaderRow(writer)
        End If
        writer.RenderEndTag()
    End Sub 'RenderHeaderRow
    
    
    
    '*********************************************************************
    '
    ' RenderNormalHeaderRow Method
    '
    ' Display all the column names.
    '
    '*********************************************************************
    Private Sub RenderNormalHeaderRow(writer As HtmlTextWriter)
        Dim col As DataColumn
        For Each col In  _dataGridData.Columns
            writer.RenderBeginTag(HtmlTextWriterTag.Td)
            writer.Write(col.ColumnName)
            writer.RenderEndTag()
        Next col
    End Sub 'RenderNormalHeaderRow
    
    
    '*********************************************************************
    '
    ' RenderSortHeaderRow Method
    '
    ' Display all the column names as links for sorting columns.
    '
    '*********************************************************************
    Private Sub RenderSortHeaderRow(writer As HtmlTextWriter)
        Dim _sortStyle As New Style()
        _sortStyle.CopyFrom(HeaderStyle)
        Dim col As DataColumn
        For Each col In  _dataGridData.Columns
            writer.RenderBeginTag(HtmlTextWriterTag.Td)
            _sortStyle.AddAttributesToRender(writer)
            RenderLink(writer, col.ColumnName, "sort_" & col.ColumnName)
            writer.RenderEndTag()
        Next col
    End Sub 'RenderSortHeaderRow
    
    
    
    '*********************************************************************
    '
    ' RenderPager Method
    '
    ' Display the pager user interface.
    '
    '*********************************************************************

    Private Sub RenderPager(writer As HtmlTextWriter)
        ' Calculate total records
        Dim totalRecords As Integer = _dataGridData.Rows.Count
        
        ' if less rows than PageSize then exit
        If _pageSize > totalRecords Then
            Return
        End If 
        ' Calculate page count
        Dim pageCount As Integer = totalRecords / _pageSize
        
        
        ' Now do a mod for any remainder
        If totalRecords Mod _pageSize > 0 Then
            pageCount += 1
        End If 
        
        
        ' calculate column count
        Dim columnCount As Integer = _dataGridData.Columns.Count
        If _enableEditing Then
            columnCount += 1
        End If 
        ' Render Pager row
        _pagerStyle.AddAttributesToRender(writer)
        writer.RenderBeginTag(HtmlTextWriterTag.Tr)
        writer.AddAttribute(HtmlTextWriterAttribute.Colspan, columnCount.ToString())
        writer.RenderBeginTag(HtmlTextWriterTag.Td)
        
        ' Render pager table
        writer.AddAttribute(HtmlTextWriterAttribute.Width, "100%")
        writer.AddAttribute(HtmlTextWriterAttribute.Cellpadding, "3")
        writer.AddAttribute(HtmlTextWriterAttribute.Cellspacing, "0")
        writer.RenderBeginTag(HtmlTextWriterTag.Table)
        writer.RenderBeginTag(HtmlTextWriterTag.Tr)
        writer.RenderBeginTag(HtmlTextWriterTag.Td)
        
        ' render previous link
        If CurrentPageIndex > 1 Then
            RenderLink(writer, _prevText, "page_" & (CurrentPageIndex - 1))
        Else
            writer.Write(_prevText)
        End If
        writer.Write("&nbsp;")
        
        ' Render each page number
        Dim pageIndex As Integer
        For pageIndex = 1 To pageCount
            writer.Write("&nbsp;")
            If pageIndex = CurrentPageIndex Then
                writer.RenderBeginTag(HtmlTextWriterTag.B)
                writer.Write(pageIndex.ToString())
                writer.RenderEndTag()
            Else
                RenderLink(writer, pageIndex.ToString(), "page_" & pageIndex.ToString())
            End If
            writer.Write("&nbsp;")
        Next pageIndex
        
        
        ' render next link
        writer.Write("&nbsp;")
        If CurrentPageIndex < pageCount Then
            RenderLink(writer, _nextText, "page_" & (CurrentPageIndex + 1).ToString())
        Else
            writer.Write(_nextText)
        End If 
        
        ' Show pager text
        writer.AddAttribute(HtmlTextWriterAttribute.Align, "right")
        writer.RenderBeginTag(HtmlTextWriterTag.Td)
        writer.Write(String.Format(_pagerText, CurrentPageIndex, pageCount))
        writer.RenderEndTag()
        
        writer.RenderEndTag() ' end cell
        writer.RenderEndTag() ' end row
        writer.RenderEndTag() ' end table
        
        writer.RenderEndTag() ' end pager row
        writer.RenderEndTag() ' end pager table
    End Sub 'RenderPager
    
    
    
    
    
    '*********************************************************************
    '
    ' RenderError Method
    '
    ' Display an error message in a JavaScript alert box.
    '
    '*********************************************************************
    Private Sub RenderError(writer As HtmlTextWriter)
        writer.AddAttribute("language", "javascript")
        writer.RenderBeginTag(HtmlTextWriterTag.Script)
        writer.Write(String.Format(" alert(""{0}""); ", _errorMessage))
        writer.RenderEndTag()
    End Sub 'RenderError
    
    
    
    '*********************************************************************
    '
    ' OnPreRender Method
    '
    ' Enable smart navigation (so the Grid will scroll back to the
    ' right place on postback) and register the control to require
    ' postback event notifications.
    '
    '*********************************************************************
    Protected Overrides Sub OnPreRender(e As EventArgs)
        Page.SmartNavigation = True
        Page.RegisterRequiresPostBack(Me)
        
        ' Get the Data
        LoadDatabaseData()
        
        ' Sort the data table
        _dataGridView = _dataGridData.DefaultView
        If _enableSorting AndAlso SortColumn <> String.Empty Then
            _dataGridView.Sort = String.Format("{0} {1}", SortColumn, SortOrder)
        End If 
        
        ' Get the values for editing
        If _editClicked Then
            Dim i As Integer
            For i = 0 To _dataGridData.Columns.Count - 1
                colEditValues(("txt" & _dataGridData.Columns(i).ColumnName)) = _dataGridView(EditItemIndex)(i)
            Next i
        End If 
        ' Update the database with any edits
        If _updateClicked Then
            UpdateDatabase()
        End If 
        ' Update the database with any deletes
        If _deleteClicked Then
            DeleteRow(_deleteItemIndex)
        End If
    End Sub 'OnPreRender
    
    
    '*********************************************************************
    '
    ' RenderContents Method
    '
    ' The main method that kicks off the rendering of the SuperDataGrid.
    '
    '*********************************************************************
    Protected Overrides Sub RenderContents(writer As HtmlTextWriter)
        
        
        ' Display any error message
        If _errorMessage <> String.Empty Then
            RenderError(writer)
        End If 
        ' Render opening table tag
        _tableStyle.AddAttributesToRender(writer)
        writer.RenderBeginTag(HtmlTextWriterTag.Table)
        
        ' Render Header Row
        RenderHeaderRow(writer)
        
        ' Render the HTML table
        RenderTable(writer)
        
        ' Render Pager
        If _enablePaging Then
            RenderPager(writer)
        End If 
        ' Render End Table Tag
        writer.RenderEndTag()
    End Sub 'RenderContents
    
    
    
    
    
    '*********************************************************************
    '
    ' SuperDataGrid Constructor
    '
    ' Set defaults for formatting (user can override).
    '
    '*********************************************************************
    Public Sub New()
        TableStyle.CellSpacing = 0
        TableStyle.CellPadding = 4
        TableStyle.GridLines = GridLines.Vertical
        TableStyle.BorderColor = ColorTranslator.FromHtml("#999999")
        TableStyle.BorderStyle = BorderStyle.Solid
        TableStyle.BorderWidth = New Unit("1px")
        
        HeaderStyle.BackColor = ColorTranslator.FromHtml("#000084")
        HeaderStyle.ForeColor = Color.White
        HeaderStyle.Font.Bold = True
        
        ItemStyle.BackColor = ColorTranslator.FromHtml("#EEEEEE")
        AlternatingItemStyle.BackColor = Color.Gainsboro
        
        PagerStyle.BackColor = ColorTranslator.FromHtml("#999999")
    End Sub 'New


End Class 'SuperDataGrid
End Namespace
